/*	
	Each of the DB enties managed by the console
	is expressed as a Javascript object, which can 
	be managed in collections, eg arrays.

	The objects with convenince methods and their
	callbacks for using JSRS are defined here
*/
	

function findByDesc(zHash, desc_tx) {

/*
	This function finds the id_pk of 
	a particular entry in the z_list 
	from its desc_tx
*/

	for (var i = 0 ; i < zHash.length() ; i++) {
		currObj = zHash.getByIndex(i);
		if (currObj.desc_tx == desc_tx) {
			return currObj.id_pk;
		}
	}
	return "null";
}
	

// The functions for persisting to db
function persistPiece(piece) {
	persistTracks(piece);
	jsrsPOST = 1;
	jsrsExecute("dbHandler.php", dbHandlerCallBack, "persistPiece", piece.toArray());
}

function createPiece(piece) {
	jsrsPOST = 1;
	jsrsExecute("dbHandler.php", insertPieceCallBack, "insertPiece", piece.toArray());
}

function persistTracks(piece) {

	var currId_pk = piece.id_pk;

	trackRow = document.getElementById("track_row_" + currId_pk);

        // Make sure each track is updated to reflect any changes to any html fields
	if (trackRow) {
		doUpdateTrackObjs(currId_pk);
	}

	currTracksHash = piece.getTracks();

	for (i = 0; i < currTracksHash.length(); i++) {
		currTrack = currTracksHash.getByIndex(i);
		jsrsPOST = 1;
		if (currTrack.id_pk.substr(0,3) == "new") {
			lastInsertedTrack = currTrack;
			jsrsExecute("dbHandler.php", insertTrackCallBack, "insertTrack", currTrack.toArray());
		} else {
			jsrsExecute("dbHandler.php", dbHandlerCallBack, "persistTrack", currTrack.toArray());
		}
	}
}

function dbHandlerCallBack(errorString) {
	if (errorString.length > 0) {
		alert("There has been an error - " + errorString);
		alert("Your changes have NOT been made!");
	}
}

function insertPieceCallBack(returnString) {

	if (returnString.substr(0, 5) == "ERROR") {
		alert("There has been an error - " + returnString);
		alert("Your changes have NOT been made!");
	} else {
		newPieceId_pk = (returnString.split("-"))[0];
		oldPieceId_pk = (returnString.split("-"))[1];
		
		currPiece = pieceHash.getById_pk(oldPieceId_pk);
		pieceHash.removeById_pk(oldPieceId_pk);
		currPiece.setNewId_pk(newPieceId_pk);
		pieceHash.add(currPiece);
		
		persistTracks(currPiece);	
		
		// Destroy and recreate the piece table
		// to ensure field names are accurate
		isCreate = false;
		doUpdatePieceFields();
	}
}

function insertTrackCallBack(returnString) {

	if (returnString.substr(0, 5) == "ERROR") {
		alert("There has been an error - " + returnString);
		alert("Your changes have NOT been made!");
	} else {

		newTrackId_pk = (returnString.split("-"))[0];
		oldTrackId_pk = (returnString.split("-"))[1];
		pieceId_pk = (returnString.split("-"))[2];

		currPiece = pieceHash.getById_pk(pieceId_pk);

		currTrack = currPiece.getTracks().getById_pk(oldTrackId_pk);
		currTrack.id_pk = newTrackId_pk;

		// Ok, now we need to ensure this update is
		// propogated to the html fields as well.
		doUpdateTrackFields(oldTrackId_pk, newTrackId_pk);
	}
}

function hash() {

/* 
	This data structure will be used to give provide the behaviour 
	of a hash table for any of our collections of objects.  They
	can be looked up by id_pk (hash[id_pk] = object) but we can 
	still iterate over the entire collection using the iterator 
	(iterator[index] = id_pk)
*/

	this.hash = new Array();
	this.iterator = new Array();

	this.add = function(newItem) {
		this.hash[newItem.id_pk] = newItem;
		this.iterator.push(newItem.id_pk);
	};

	this.getById_pk = function(id_pk) {
		return this.hash[id_pk];
	};

	this.getByIndex = function(i) {
		return this.hash[this.iterator[i]];
	};

	this.length = function() {
		return this.iterator.length;
	};

	this.removeById_pk = function(id_pk) {
		// chop out the item from both arrays
		this.newHash = new Array();
		this.newIterator = new Array();
		j = 0;
		for (i = 0; i < this.iterator.length; i++) {
			currItem = this.hash[this.iterator[i]];
			if (currItem.id_pk != id_pk) {
				this.newHash[currItem.id_pk] = currItem;
				this.newIterator.push(currItem.id_pk);
			}
		}
		this.hash = this.newHash;
		this.iterator = this.newIterator;
	};

}
		
function pieceObj(id_pk, z_format_fk, z_country_fk, vjcatno_tx, artist_tx, title_tx, label_tx, catno_tx, year_nr, category_tx, audio_bl, country_tx, price_nr, new_bl, vjchoice_bl, img_bl, tracklist_bl, active_bl, qty_nr, notes_tx, review_tx, rare_bl, sale_bl, distributor_fk, outofstock_bl) {

/* 
	This object represents the db object Piece.  Most DB fields
	are expressed, the left out fields are either not relavent to 
	the web admin - TIMESTAMP_DT - or are not used, the rest of the
	left out ones.  The database will be cleaned of these fields
	during redevelopment.

*/

	this.id_pk = id_pk;
	this.z_format_fk = z_format_fk;
	this.z_country_fk = z_country_fk;
	this.vjcatno_tx = vjcatno_tx;
	this.artist_tx = artist_tx;
	this.title_tx = title_tx;
	this.label_tx = label_tx;
	this.catno_tx = catno_tx;
	this.year_nr = year_nr;
	this.category_tx = category_tx;
	this.audio_bl = audio_bl;
	this.country_tx = country_tx; 
	this.price_nr = price_nr;
	this.new_bl = new_bl;
	this.vjchoice_bl = vjchoice_bl;
	this.img_bl = img_bl;
	this.tracklist_bl = tracklist_bl;
	this.active_bl = active_bl;
	this.qty_nr = qty_nr;
	this.notes_tx = notes_tx;
	this.review_tx = review_tx;
	this.rare_bl = rare_bl;
	this.sale_bl = sale_bl;
	this.distributor_fk = distributor_fk;
	this.outofstock_bl = outofstock_bl;

	this.toArray = function () {
		return (new Array(
			this.id_pk + '',
			this.z_format_fk + '',
			this.z_country_fk + '',
		   this.vjcatno_tx + '',
		   this.artist_tx + '',
		   this.title_tx + '',
		   this.label_tx + '',
		   this.catno_tx + '',
		   this.year_nr + '',
		   this.category_tx + '',
		   this.audio_bl + '',
		   this.country_tx + '',
		   this.price_nr + '',
			this.new_bl + '',
			this.vjchoice_bl + '',
			this.img_bl + '',
			this.tracklist_bl + '',
			this.active_bl + '',
			this.qty_nr,
			this.notes_tx + '',
			this.review_tx + '',
			this.rare_bl + '',
			this.sale_bl + '',
			this.distributor_fk + '',
			this.outofstock_bl + ''));
	};

	var tracks = new hash();

	this.addTrack = function(newTrack) {
		tracks.add(newTrack);
	};

	this.getTracks = function() {
		return tracks;
	};

	this.hasTracks = function() {
		if (tracks.length() > 0) { return true; }
		else { return false; }
	};

	this.numberOfTracks = function() {
		return tracks.length();
	};
	
	this.setNewId_pk = function(newId_pk) {
		this.id_pk = newId_pk;

		//update the track records with new id_pk
		// and vjsubcatno_tx
		for (i = 0; i < tracks.length(); i++) {
			currTrack = tracks.getByIndex(i);
			currTrack.piece_fk = newId_pk;
			currTrack.vjsubcatno_tx = this.vjcatno_tx + currTrack.vjsubcatno_tx.substring(currTrack.vjsubcatno_tx.indexOf("_"));
		}
	};
}


function trackObj(id_pk, piece_fk, vjsubcatno_tx, title_tx, artist_tx, audio_bl, side_tx) {

/*
	This object represents a Track, and expressed all 
	track DB fields except ARTIST_ALIAS_FK which is not 
	used.

*/

	this.id_pk = id_pk;
	this.piece_fk = piece_fk;
	this.vjsubcatno_tx = vjsubcatno_tx;
	this.title_tx = title_tx;
	this.artist_tx = artist_tx;
	this.audio_bl = audio_bl;
	this.side_tx = side_tx;

	this.toArray = function() {
		return (new Array(this.id_pk + '',
				  this.piece_fk + '',
				  this.vjsubcatno_tx + '',
				  this.title_tx + '',
				  this.artist_tx + '',
				  this.audio_bl + '',
				  this.side_tx + ''));
	};

}

function z_listObj(id_pk, desc_tx) {
	
/* 
	we can use the same base object for all z_lists
*/
	
	this.id_pk = id_pk;
	this.desc_tx = desc_tx;
}
