/*
	This file contains the main Javascript
	for the operation of the console.  The 
	application behaviours are expressed here
*/

function doSavePiece(currId_pk, isCreate) {

	/* Updates the attributes of the pieceObj specified by its id_pk
	with the current values of the html fields for that piece.
	Then the piece is saved to the db via jsrs.
	Once the piece is saved the tracks are updated to
	reflect any changes in their fields and saved to 
	the db one by one via jsrs
	*/

	var currPiece = pieceHash.getById_pk(currId_pk);
	
	// Update the piece Object to reflect the new values
	formatObj = gE("piece_z_format_fk_" + currId_pk);
	formatDesc = formatObj.options[formatObj.selectedIndex].value;
	currPiece.z_format_fk = findByDesc(z_formatHash, formatDesc);
	
	// 2 fields to populate for country for legacy
	countryObj = gE("piece_z_country_fk_" + currId_pk);
	countryDesc = countryObj.options[countryObj.selectedIndex].value;
	currPiece.z_country_fk = findByDesc(z_countryHash, countryDesc);
	currPiece.country_tx = countryDesc;	
	
	currPiece.vjcatno_tx = gE("piece_vjcatno_tx_" + currId_pk).value;
	currPiece.artist_tx = gE("piece_artist_tx_" + currId_pk).value;
	currPiece.title_tx = gE("piece_title_tx_" + currId_pk).value;
	currPiece.label_tx = gE("piece_label_tx_" + currId_pk).value;
	currPiece.catno_tx = gE("piece_catno_tx_" + currId_pk).value;
	currPiece.year_nr = gE("piece_year_nr_" + currId_pk).value;
	currPiece.category_tx = gE("piece_category_tx_" + currId_pk).value;

	audioValue = gE("piece_audio_bl_" + currId_pk).checked;	
	if (audioValue == true) {
		currPiece.audio_bl = 1;
	} else {
		currPiece.audio_bl = 0;
	}
	
	currPiece.price_nr = gE("piece_price_nr_" + currId_pk).value;

	newValue = gE("piece_new_bl_" + currId_pk).checked;
	if (newValue == true) {
		currPiece.new_bl = 1;
	} else {
		currPiece.new_bl = 0;
	}

	vjchoiceValue = gE("piece_vjchoice_bl_" + currId_pk).checked;
	if (vjchoiceValue == true) {
		currPiece.vjchoice_bl = 1;
	} else {
		currPiece.vjchoice_bl = 0;
	}

	imgValue = gE("piece_img_bl_" + currId_pk).checked;
	if (imgValue == true) {
		currPiece.img_bl = 1;
	} else {
		currPiece.img_bl = 0;
	}

	tracklistValue = gE("piece_tracklist_bl_" + currId_pk).checked;
	if (tracklistValue == true) {
		currPiece.tracklist_bl = 1;
	} else {
		currPiece.tracklist_bl = 0;
	}

	activeValue = gE("piece_active_bl_" + currId_pk).checked;
	if (activeValue == true) {
		currPiece.active_bl = 1;
	} else {
		currPiece.active_bl = 0;
	}
	
	currPiece.qty_nr = gE("piece_qty_nr_" + currId_pk).value;

	notesField = gE("piece_notes_tx_" + currId_pk);
	if (notesField) {
		currPiece.notes_tx = notesField.value;
	}
	
	reviewField = gE("piece_review_tx_" + currId_pk);
	if (reviewField) {
		currPiece.review_tx = reviewField.value;
	}
	
	rareValue = gE("piece_rare_bl_" + currId_pk).checked;
	if (rareValue == true) {
		currPiece.rare_bl = 1;
	} else {
		currPiece.rare_bl = 0;
	}
	
	saleValue = gE("piece_sale_bl_" + currId_pk).checked;
	if (saleValue == true) {
		currPiece.sale_bl = 1;
	} else {
		currPiece.sale_bl = 0;
	}
	
	if (isCreate) {
		createPiece(currPiece);
	} else {
		persistPiece(currPiece);
	}

	trackRow = gE("track_row_" + currId_pk);

	// Make sure each track is updated to reflect any changes to any html fields
	if (trackRow) {
		doUpdateTrackObjs(currId_pk);
	}

	persistTracks(currPiece);
}

function doSaveAll() {

	// Saves all pieces in pieceHash one by one

	for (i = 0; i < pieceHash.length(); i++) {
		doSavePiece(pieceHash.getByIndex(i).id_pk);
	}
}

function doUpdateTrackObjs(currId_pk) {

	/* Updates the object attributes of each trackObj
	belonging to the current pieceObj with the values of 
	the appropriate HTML fields*/
	
	 currPiece = pieceHash.getById_pk(currId_pk);
	 currTrackHash = currPiece.getTracks();
	 for (i = 0; i < currTrackHash.length(); i++) {
	 	 currTrack = currTrackHash.getByIndex(i);
		 currTrackId_pk = currTrack.id_pk;
        	 currTrack.vjsubcatno_tx = currPiece.vjcatno_tx + "_" + gE("track_vjsubcatno_tx_" + currTrackId_pk).value;
	         currTrack.title_tx = gE("track_title_tx_" + currTrackId_pk).value;
        	 currTrack.artist_tx = gE("track_artist_tx_" + currTrackId_pk).value;
                                                                                                                                                               
	         audioValue = gE("track_audio_bl_" + currTrackId_pk).checked;
        	 if (audioValue == true) {
			currTrack.audio_bl = 1;
	         } else {
        	        currTrack.audio_bl = 0;
	         }
        	 currTrack.side_tx = gE("track_side_tx_" + currTrackId_pk).value;
	}
}

function doUpdatePieceFields() {

	gE("tablediv").removeChild(gE("master_table"));
	buildTable();

}

function doUpdateTrackFields(oldTrackId_pk, newTrackId_pk) {

	gE("track_vjsubcatno_tx_" + oldTrackId_pk).id = "track_vjsubcatno_tx_" + newTrackId_pk;
	gE("track_title_tx_" + oldTrackId_pk).id = "track_title_tx_" + newTrackId_pk;
	gE("track_artist_tx_" + oldTrackId_pk).id = "track_artist_tx_" + newTrackId_pk;
	gE("track_audio_bl_" + oldTrackId_pk).id = "track_audio_bl_" + newTrackId_pk;
	gE("track_side_tx_" + oldTrackId_pk).id = "track_side_tx_" + newTrackId_pk;

}

function doCreateTrack(currId_pk) {

	/* Creates a new track object using the 
	values of the html fields in the insert row
	to populate the attributes of the object */

	currPiece = pieceHash.getById_pk(currId_pk);
   audioValue = gE("track_audio_bl_new_" + currId_pk).checked;
   if (audioValue == true) {
    		audioBit = 1;
	} else {
		audioBit = 0;
	}
	
	// Note adding numberOfTracks to new, this ensures all new tracks
	// have a unique temporary id_pk
	newTrack = new trackObj("new_" + currPiece.numberOfTracks(),
			     currId_pk,
			     currPiece.vjcatno_tx + "_" + (currPiece.numberOfTracks() + 1),
			     gE("track_title_tx_new_" + currId_pk).value,
			     gE("track_artist_tx_new_" + currId_pk).value,
			     audioBit,
			     gE("track_side_tx_new_" + currId_pk).value);
	
	doCloseTracks(currId_pk);
	
	currPiece.addTrack(newTrack);
	
	tracksHash = currPiece.getTracks();
	
	for (i = 0; i < tracksHash.length(); i++) {
		currTrack = tracksHash.getByIndex(i);
	}
	
	doShowTracks(currId_pk);
}

function doShowTracks(currId_pk) {

	/* Inserts the html necessary to display
	the trackObj records for a particular piece */

	currPiece = pieceHash.getById_pk(currId_pk);
	trackDiv = getLayerObj("track_div_" + currId_pk);
	tracksHash = currPiece.getTracks();
	
	// Prepare a table for insertion inside a cell
	htmlString = "<td align=\"center\"><table id=\"track_table_" + currId_pk + "\" border=\"2\" width=\"100%\"><tbody>";

	// With a header row
	htmlString += "<tr bgcolor=\"#FF9900\">";
	htmlString += "<td align=\"center\" colspan=\"3\">VJ Track No</td>";
	htmlString += "<td align=\"center\">Artist</td>";
	htmlString += "<td align=\"center\">Title</td>";
	htmlString += "<td align=\"center\">Side</td>";
	htmlString += "<td align=\"center\">Audio</td>";
	htmlString += "</tr>";

	// And a Row for adding new tracks
	htmlString += "<tr>";
	htmlString += "<td><button id=\"newTrackButton_" + currId_pk + "\" type=\"button\" value=\"NewTrack\" onclick=\"doCreateTrack(" + currId_pk + ")\">Insert</button></td>";
	htmlString += "<td align=\"center\" colspan=\"2\">" + currPiece.vjcatno_tx + "_" + (tracksHash.length() + 1) + "</td>";
	htmlString += createTextField("track_artist_tx_new_" + currId_pk, 20, 1, "");
	htmlString += createTextField("track_title_tx_new_" + currId_pk, 20, 1, "");
	htmlString += createTextField("track_side_tx_new_" + currId_pk, 18, 1, "");
	htmlString += createChb("track_audio_bl_new_" + currId_pk, "");
	htmlString += "</tr>";
	
	htmlString += "<tr><td colspan=\"7\">&nbsp;</td></tr>";

	// Then a row for each track
	for (i = 0; i < tracksHash.length(); i++) {
		currTrack = tracksHash.getByIndex(i);
		trackId_pk = currTrack.id_pk;
		htmlString += "<tr>";

		htmlString += "<td><button id=\"delTrackButton_" + trackId_pk + "\" type=\"button\" value=\"DelTrack\" onclick=\"doDelTrack(" + currId_pk + ", " + trackId_pk + ")\">Delete</button></td>";
		htmlString += createLabelCell(currTrack.vjsubcatno_tx.substring(0, currTrack.vjsubcatno_tx.indexOf("_") + 1));
		htmlString += createTextField("track_vjsubcatno_tx_" + trackId_pk, 2, 1, currTrack.vjsubcatno_tx.substring(currTrack.vjsubcatno_tx.indexOf("_") + 1));
		htmlString += createTextField("track_artist_tx_" + trackId_pk, 20, 1, currTrack.artist_tx);
		htmlString += createTextField("track_title_tx_" + trackId_pk, 20, 1, currTrack.title_tx);
		htmlString += createTextField("track_side_tx_" + trackId_pk, 18, 1, currTrack.side_tx);
		htmlString += createChb("track_audio_bl_" + trackId_pk, currTrack.audio_bl);

		htmlString += "</tr>";
	}	
	htmlString += "</table></td>";

	alert(htmlString);

	setLayerHTML(trackDiv, htmlString);

	tracksButtonDiv = getLayerObj("tracksButtonDiv_" + currId_pk);
	htmlString = "<button id=\"tracksButton_" + currId_pk + "\" value=\"CloseTracks\" type=\"button\" onclick=\"doCloseTracks(" + currId_pk + ")\">Close Tracks</button>";
	setLayerHTML(tracksButtonDiv, htmlString);

}

function doCloseTracks(currId_pk) {

	// Removes the tracks table from the display

	// update the track object before closing so that data is not lost
	doUpdateTrackObjs(currId_pk);

	// Then close the row
	trackDiv = getLayerObj("track_div_" + currId_pk);
	setLayerHTML(trackDiv, "");

	tracksButtonDiv = getLayerObj("tracksButtonDiv_" + currId_pk);
	htmlString = "<button id=\"tracksButton_" + currId_pk + "\" value=\"Tracks\" type=\"button\" onclick=\"doShowTracks(" + currId_pk + ")\">Tracks</button>";
	setLayerHTML(tracksButtonDiv, htmlString);
}	

/*
	doShowNotes has been retrofitted to handle
	the review field as well.  They will be treated 
	as a group in all code.
*/

function doShowNotes(currId_pk) {

	// Displays the notes field

	currPiece = pieceHash.getById_pk(currId_pk);
        notesDiv = getLayerObj("notes_div_" + currId_pk);
	
	showLayer(notesDiv);

	notesButtonDiv = getLayerObj("notesButtonDiv_" + currId_pk);
	htmlString = "<button id=\"notesButton_" + currId_pk + "\" value=\"Notes\" type=\"button\" onclick=\"doCloseNotes(" + currId_pk + ")\">Close Text</button>";

	setLayerHTML(notesButtonDiv, htmlString);

}

/*
	doCloseNotes has been retrofitted to handle
	the review field also.
*/

function doCloseNotes(currId_pk) {

	// Closes the notes field

	notesDiv = getLayerObj("notes_div_" + currId_pk);
	hideLayer(notesDiv);
	setLayerSize(notesDiv, 0, 0);

	 notesButtonDiv = getLayerObj("notesButtonCell_" + currId_pk);
	 htmlString = "<button id=\"notesButton_" + currId_pk + "\" value=\"Text\" type=\"button\" onclick=\"doShowNotes(" + currId_pk + ")\">Notes</button>";
	setLayerHTML(notesButtonDiv, htmlString);
}

function doBulkChange(fld) {

	/* Updates the html fields of all the pieces 
	the current table to the current value of the 
	specified field.  This allows bulk changes to be made to
	certain fields */

	if (fld == "price_nr") {
		for (i = 0; i < pieceHash.length(); i++) {
			currPiece = pieceHash.getByIndex(i);
			currId_pk = currPiece.id_pk;
			gE("piece_price_nr_" + currId_pk).value = gE("piece_price_nr_bulk").value;
		}
		
	} else {
		for (i = 0; i < pieceHash.length(); i++) {
			currPiece = pieceHash.getByIndex(i);
			currId_pk = currPiece.id_pk;
			gE("piece_" + fld + "_" + currId_pk).checked = gE("piece_" + fld + "_bulk").checked;
		}
	}
}

function doRemovePiece(currId_pk) {

	/* Removes a piece from the table.  This allows
	the data set to be reduced before a bulk update 
	for instance */

	pieceTable = gE("piece_table_" + currId_pk);
	pieceTable.parentNode.removeChild(pieceTable);

	pieceHash.removeById_pk(currId_pk);

}

function doBuildPrintReport() {

	document.body.background = "";
	document.body.bgcolor = "#000000";

	wH(gE("divButtonPrintReport"), "<button onclick=\"buildTable()\">Return</button>");

	htmlString = "<br><table id=\"print_table\" width=\"780\" border=\"1\"><thead><tr>";

	htmlString += "<td align=\"center\"><strong>VJ Num</strong></td>";
	htmlString += "<td align=\"center\"><strong>Cat Num</strong></td>";
	htmlString += "<td><strong>Artist</strong></td>";
	htmlString += "<td><strong>Title</strong></td>";
	htmlString += "<td><strong>Label</strong></td>";
	htmlString += "<td><strong>Format</strong></td>";
	htmlString += "<td><strong>Country</strong></td>";
	htmlString += "<td align=\"center\"><strong>&pound;</strong></td>";
	htmlString += "<td align=\"center\"><strong>Qty</strong></td>";

	htmlString += "</tr></thead><tbody>";

	for(i = 0; i < pieceHash.length(); i++) {
		currPiece = pieceHash.getByIndex(i);
		
		htmlString += "<tr>";

		htmlString += "<td align=\"center\">" + currPiece.vjcatno_tx + "</td>";
		htmlString += "<td align=\"center\">" + currPiece.catno_tx + "</td>";
		htmlString += "<td align=\"left\">" + currPiece.artist_tx + "</td>";
		htmlString += "<td align=\"left\">" + currPiece.title_tx + "</td>";
		htmlString += "<td align=\"left\">" + currPiece.label_tx + "</td>";
		htmlString += "<td align=\"left\">" + z_formatHash.getById_pk(currPiece.z_format_fk).desc_tx + "</td>";
		htmlString += "<td align=\"left\">" + currPiece.country_tx + "</td>";
		htmlString += "<td align=\"center\">" + currPiece.price_nr + "</td>";
		if (currPiece.qty_nr) {
			htmlString += "<td align=\"center\">" + currPiece.qty_nr + "</td>";
		} else {
			htmlString += "<td>&nbsp;</td>";
		}

		htmlString += "</tr>";

	}

	htmlString += "</tbody></table>";

	wH(gE("tablediv"), htmlString);
}
